#include "ConfigFile.h"

//*******************************************************************************************************************//
//For example:
//If you want to get "AutoMode type" from configuration file. You can use GetInt ("AutoMode");
//and the return value is the AutoMode type value. If you want to get the "ExternalCommand" from configuration file,
//you can use GetString("ExternalCommand", szCmdBuf), and the buffer szCmdBuf contain the external command string.
//*******************************************************************************************************************//

CConfigFile::CConfigFile()
{
	m_pFilePointer = NULL;
	memset(&m_ShutdownParam, 0, sizeof(m_ShutdownParam));
	GetShutdownParameter(&m_ShutdownParam);
}

CConfigFile::~CConfigFile()
{
}

//********************************************//
//  Parameters:
//      pszFileName
//          [in] File path.
//      pszMode
//          [in] Open file mode.
//  Return value:
//      Succeed, return 1; failed, return 0.
//  Description:
//      Open the file with a certain mode.
//********************************************//
int CConfigFile::OpenFile(char *pszFileName, char *pszMode)
{
	if ((m_pFilePointer = fopen(pszFileName, pszMode)) == NULL)
		return 0;

	return 1;
}

//************************************************************************//
//  Return value:
//      If the configuration file descriptor is not available, return 0.
//  Description:
//      Close configuration file.
//************************************************************************//
int CConfigFile::CloseFile()
{
	if (m_pFilePointer != NULL)
	{
		fclose(m_pFilePointer);
		m_pFilePointer = NULL;
		return 1;
	}

	return 0;
}

int CConfigFile::FindSection(char *pszSection)
{
	int iLen = 0;
	char szBuf[MAX_LINELENGTH];
	memset(szBuf, 0, sizeof(szBuf));

	if ((pszSection == NULL) || (m_pFilePointer == NULL))
		return 0;

	fseek(m_pFilePointer, 0L, SEEK_SET);

	while (fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))
	{
		iLen = strlen(szBuf);
		if (iLen > 1)
			szBuf[iLen-1] = 0;
		if (strcmp(szBuf, pszSection) == 0)
			return 1;

		memset(szBuf, 0, sizeof(szBuf));
	}

	return 0;
}

//********************************************************************//
//  Parameters:
//      pszItem
//          [in] The item whose value want to get.
//      pszItemVal
//          [out] The string value of the item.
//  Return value:
//      If get the string, return 1; otherwise return -1.
//  Description:
//      According to the item (pszItem), get the item related string.
//********************************************************************//
int CConfigFile::GetString(char *pszItem, char *pszItemVal)
{
	char szBuf[MAX_LINELENGTH];
	char szCurItem[MAX_LINELENGTH];

	do
	{
		memset(szBuf, 0, sizeof(szBuf));
		memset(szCurItem, 0, sizeof(szCurItem));
		if (!fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))
			return -1;

		if (!GetItemFromLine(szBuf, szCurItem, pszItemVal))
			return -1;

	} while(strcmp(pszItem, szCurItem));

	return 1;
}

//********************************************************************//
//  Parameters:
//      pszItem
//          [in] The item whose value want to get.
//  Return value:
//      Return value of the item (pszItem pointed).
//  Description:
//      According to the item (pszItem)， get the item related number.
//********************************************************************//
int CConfigFile::GetInt(char *pszItem)
{
	char szBuf[MAX_LINELENGTH];
	char szCurItem[MAX_LINELENGTH];
	char szCurItemValue[MAX_LINELENGTH];

	do
	{
		memset(szBuf, 0, sizeof(szBuf));
		memset(szCurItem, 0, sizeof(szCurItem));
		memset(szCurItemValue, 0, sizeof(szCurItemValue));

		if (!fgets(szBuf, MAX_LINELENGTH, m_pFilePointer))
			return -1;
		if (!GetItemFromLine(szBuf, szCurItem, szCurItemValue))
			return -1;

	} while(strcmp(pszItem, szCurItem));

	return	atoi(szCurItemValue);
}

//*****************************************************//
//  Parameters:
//      pszBuf
//          [in] string buffer.
//      szItem
//          [out] Item in the buffer.
//      szItemValue
//          [out] item value in the buffer.
//  Return value:
//      If get the Item value succeed, return 1.
//  Description:
//      Get item and item value in the buffer (pszBuf).
//*****************************************************//
int CConfigFile::GetItemFromLine(char *pszBuf, char *pszItem, char *pszItemValue)
{
	UINT nStartPos = 0;
	char szTemp[MAX_LINELENGTH];

	memset(szTemp, 0, sizeof(szTemp));
	GetStringFromLine(pszItem, pszBuf, &nStartPos, '=');

	if (pszItem == NULL)
		return 1;

	if (pszItem[0] == '[')
		return 0;

	if (nStartPos == ENDOFLINE)
		return 1;

	GetStringFromLine(szTemp, pszBuf, &nStartPos, '\n');

	if (strlen(szTemp) >= 256)
		return 0;

	strcpy(pszItemValue, szTemp);

	return 1;
}

//**********************************************************************************************************************************//
//  Parameters:
//      pszDest
//          [out] Storage location of the destination string.
//      pszBuf
//          [in] String buffer;
//      nStartPos
//          [in/out] Start offset of the string buffer;
//      szSeparator
//          [in] Separator character of the line.
//  Return value:
//      If not arrive at the end of the file, return 1.
//  Description:
//      Get all the characters before the separator (szSeparator) character in the buffer (pszBuf), and trim the destination string.
//**********************************************************************************************************************************//
int CConfigFile::GetStringFromLine(char *pszDest, char *pszBuf, UINT *nStartPos, char szSeparator)
{
	char szCur;
	UINT nPos;
	char szTemp[MAX_LINELENGTH];
	int	i = 0;

	memset(szTemp, 0, sizeof(szTemp));
	nPos = *nStartPos;

	if (nPos == ENDOFLINE)
		return 0;

	szCur = pszBuf[nPos];

	while ((szCur != szSeparator) && (szCur != '\0'))
	{
		szTemp[i++] = szCur;
		nPos++;
		szCur = pszBuf[nPos];
	}	

	if (szCur == '\n')
		nPos = ENDOFLINE;
	else
		nPos++;

	memset(pszDest, 0, MAX_LINELENGTH);
	TrimString(pszDest, szTemp);
	*nStartPos = nPos;

	return 1;
}

//************************************************************************************************************//
//  Parameters:
//      pszDest
//          [out] Storage location of the trimmed string.
//      pszStr
//          [in] Storage location of the string to be trimmed.
//  Return value:
//      Return 0, if the string does not contain characters except space and tab character; otherwise return 1.
//  Description:
//      Delete the space and tab key in the string (pszStr).
//************************************************************************************************************//
int CConfigFile::TrimString(char *pszDest, char *pszStr)
{
	int iStartPos = 0, iEndPos = 0;
	int iLength = 0;
	char *ptr = NULL;

	iLength = strlen(pszStr);
	iEndPos = iLength;

	if (iLength == 0)
	{
		pszDest[0] = '\0';
		return 0;
	}

	ptr = pszStr;
	while ((*ptr != '\0') && (*ptr != '\n'))
	{
		if ((*ptr == 32) || (*ptr == '\t'))
		{
			iStartPos++;
			ptr++;
		}
		else
			break;
	}

	ptr = pszStr;
	while ((*ptr != '\0') && (*ptr != '\n'))
	{
		ptr++;
	}
	ptr--;

	while ((*ptr != '\0') && (*ptr != '\n'))
	{
		if ((*ptr == 32) || (*ptr == '\t'))
		{
			iEndPos--;
			ptr--;
		}
		else
			break;
	}

	if (iEndPos > iStartPos)
		memcpy(pszDest, pszStr+iStartPos, iEndPos-iStartPos);
	else
	{
		pszDest[0] = '\0';
		return 0;
	}

	return 1;
}

//***************************************************************************************************//
//  Parameters:
//      pShutdownParam
//          [out] storage location of the got shutdown parameter.
//  Return value:
//      If open configuration file failed, or the parameter is NULL, return 0, otherwise return 1.
//  Description:
//      Read shutdown parameter from configuration file.
//***************************************************************************************************//
int CConfigFile::GetShutdownParameter(PSHUTDOWNPARAM pShutdownParam)
{
	int	iRtn = 0;
	int	iValue = 0;
	char szValue[MAX_LINELENGTH];
	int	iFilePointer = 0;

	memset(szValue, 0, sizeof(szValue));

	if (OpenFile(CONFIG_FILENAME, "r") == 0)
	{
		SetDefaultShutdownParam(&m_ShutdownParam);
		WriteConfigurationFile();
		return 0;
	}

	if ((m_pFilePointer == NULL) || (pShutdownParam == NULL))
		iRtn = 0;

	if (FindSection(ShutdownParamSection))
		iRtn = 1;

	if (iRtn)
	{
		iFilePointer = ftell(m_pFilePointer);
		fseek(m_pFilePointer, iFilePointer, SEEK_SET);

		if ((iValue = GetInt(S_AutoMode)) == -1)
			pShutdownParam->AutoMode = SELECT_MODE;
		else
			pShutdownParam->AutoMode = iValue;

		//PortType
		if ((iValue = GetString(S_PortType, szValue)) == -1)
			pShutdownParam->uPortType = PORTTYPE_DEFAULT;
		else
		{
			if (!strcmp(PORTSTR_DEFAULT, szValue))
				pShutdownParam->uPortType = PORTTYPE_DEFAULT;
			else
				pShutdownParam->uPortType = PORTTYPE_USB;
		}

		//PortNum
		if ((iValue = GetInt(S_PortNum)) == -1)
			pShutdownParam->uComPort = COMPORT_DEFAULT;
		else
			pShutdownParam->uComPort = iValue;

		//AC fail delay time
		memset(szValue, 0, sizeof(szValue));
		if ((iValue = GetInt(S_ACFailDelayTime)) == -1)
			pShutdownParam->uACFailDelayTime = ACFAILDELAY_DEFAULT;
		else
			pShutdownParam->uACFailDelayTime = iValue;

		//External Command
		if ((iValue = GetString(S_ExternCmd, szValue)) == -1)
			memcpy(&(pShutdownParam->szExternCmd), EXTERNCMD_DEFAULT, 1);
		else
		    memcpy(&(pShutdownParam->szExternCmd), szValue, sizeof(szValue));

		//External command need time
		if ((iValue = GetInt(S_ExCmdNeedTime)) == -1)
			pShutdownParam->uExCmdNeedTime = EXCMDNEEDTIME_DEFAULT;
		else
			pShutdownParam->uExCmdNeedTime = iValue;

		//Shutdown need time
		if ((iValue = GetInt(S_ShutdownNeedTime)) == -1)
			pShutdownParam->uShutdownNeedTime = SDNEEDTIME_DEFAULT;
		else
			pShutdownParam->uShutdownNeedTime = iValue;

		//seng message choice
		if ((iValue = GetInt(S_SendMsg)) == -1)
			pShutdownParam->iSendMsg = SENDMSG_DEFAULT;
		else
			pShutdownParam->iSendMsg = iValue;
		
		//Ups Communication Mode choice
		if ((iValue = GetInt(S_UpsCommMode)) == -1)
			pShutdownParam->iUpsCommMode = UpsCommMode_DEFAULT;
		else
			pShutdownParam->iUpsCommMode = iValue;

		//Ups Shutdown Mode choice
		if ((iValue = GetInt(S_ShutdownMode)) == -1)
			pShutdownParam->iShutdownMode= ShutdownMode_DEFAULT;
		else
			pShutdownParam->iShutdownMode= iValue;

		//KVM support choice
		if ((iValue = GetInt(S_KVMSupport)) == -1)
			pShutdownParam->iKVMMode= KVMMode_DEFAULT;
		else
			pShutdownParam->iKVMMode= iValue;

		//GuestOSMode allen20110916
		if ((iValue = GetInt(S_GuestOSMode)) == -1)
			pShutdownParam->iGuestOSMode= GuestOSMode_DEFAULT;
		else
			pShutdownParam->iGuestOSMode= iValue;
	}
	else
		SetDefaultShutdownParam(pShutdownParam);

	CloseFile();
	return iRtn;
}

//************************************************************************//
//  Parameters:
//      pShutdownParam
//          [in] storage location of the setted shutdown parameter.
//  Return value:
//      None.
//  Description:
//      Set the shutdown parameter (pShutdownParam) to the default value.
//************************************************************************//
void CConfigFile::SetDefaultShutdownParam(PSHUTDOWNPARAM pShutdownParam)
{
	if (pShutdownParam != NULL)
	{
		pShutdownParam->AutoMode = SELECT_MODE;
		pShutdownParam->uPortType = PORTTYPE_DEFAULT;
		pShutdownParam->uComPort = COMPORT_DEFAULT;
		pShutdownParam->uACFailDelayTime = ACFAILDELAY_DEFAULT;
		memcpy(&(pShutdownParam->szExternCmd), EXTERNCMD_DEFAULT, 1);
		pShutdownParam->uExCmdNeedTime = EXCMDNEEDTIME_DEFAULT;
		pShutdownParam->uShutdownNeedTime = SDNEEDTIME_DEFAULT;
		pShutdownParam->iSendMsg = SENDMSG_DEFAULT;
		pShutdownParam->iUpsCommMode = UpsCommMode_DEFAULT;
		pShutdownParam->iShutdownMode = ShutdownMode_DEFAULT;
		pShutdownParam->iKVMMode = KVMMode_DEFAULT;
		pShutdownParam->iGuestOSMode = GuestOSMode_DEFAULT; //allen20110916
	}
}

//***********************************************************************************************************//
//  Parameters:
//      ShutdownParam
//          [in] the shutdown parameter stored into configuration file.
//  Return value:
//      If write to configuration file failed, return 0; otherwise return 1.
//  Description:
//      Invoke function WriteConfigurationFile() to keep the shutdown parameter into the configuration file.
//***********************************************************************************************************//
int CConfigFile::SetShutdownParameter(SHUTDOWNPARAM ShutdownParam)
{
	m_ShutdownParam = ShutdownParam;
	return WriteConfigurationFile();
}

//*************************************************************//
//  Return value:
//      If the file can not be opened or created, return 0.
//  Description:
//      Keep the shutdown parameter to the configuration file.
//*************************************************************//
int CConfigFile::WriteConfigurationFile()
{
	char szTemp[MAX_LINELENGTH + MAX_LINELENGTH];
	char szBuf[MAX_LINELENGTH + MAX_LINELENGTH];

	if (OpenFile(CONFIG_FILENAME, "w+") == 0)
		return 0;

	WriteOneLine(ShutdownParamSection, 1);

	//SelectMode
	sprintf(szBuf, " = %d", m_ShutdownParam.AutoMode);
	sprintf(szTemp, "%s%s", S_AutoMode, szBuf);
	WriteOneLine(szTemp, 1);

	//PortType
	if (m_ShutdownParam.uPortType == PORTTYPE_DEFAULT)
		sprintf(szBuf, " = %s", PORTSTR_DEFAULT);
	else
		sprintf(szBuf, " = %s", PORTSTR_USB);

	sprintf(szTemp, "%s%s", S_PortType, szBuf);
	WriteOneLine(szTemp, 1);

	//PortNum
	sprintf(szBuf, " = %d", m_ShutdownParam.uComPort);
	sprintf(szTemp, "%s%s", S_PortNum, szBuf);
	WriteOneLine(szTemp, 1);

	//AC fail
	sprintf(szBuf, " = %d", m_ShutdownParam.uACFailDelayTime);
	sprintf(szTemp, "%s%s", S_ACFailDelayTime, szBuf);
	WriteOneLine(szTemp, 1);

	//ExternCmd Line
	sprintf(szTemp, "%s = %s", S_ExternCmd, m_ShutdownParam.szExternCmd);
	WriteOneLine(szTemp, 1);

	//Extern time
	sprintf(szBuf, " = %d", m_ShutdownParam.uExCmdNeedTime);
	sprintf(szTemp, "%s%s", S_ExCmdNeedTime, szBuf);
	WriteOneLine(szTemp, 1);

	//OS time
	sprintf(szBuf, " = %d", m_ShutdownParam.uShutdownNeedTime);
	sprintf(szTemp, "%s%s", S_ShutdownNeedTime, szBuf);
	WriteOneLine(szTemp, 1);

	//Send Message
	sprintf(szBuf, " = %d", m_ShutdownParam.iSendMsg);
	sprintf(szTemp, "%s%s", S_SendMsg, szBuf);
	WriteOneLine(szTemp, 1);

	//UpsCommMode
	sprintf(szBuf, " = %d", m_ShutdownParam.iUpsCommMode);
	sprintf(szTemp, "%s%s", S_UpsCommMode, szBuf);
	WriteOneLine(szTemp, 1);

	//Shutdown Mode
	sprintf(szBuf, " = %d", m_ShutdownParam.iShutdownMode);
	sprintf(szTemp, "%s%s", S_ShutdownMode, szBuf);
	WriteOneLine(szTemp, 1);

	//KVM support
	sprintf(szBuf, " = %d", m_ShutdownParam.iKVMMode);
	sprintf(szTemp, "%s%s", S_KVMSupport, szBuf);
	WriteOneLine(szTemp, 1);
	
	//Guest OS allen20110916
	sprintf(szBuf, " = %d", m_ShutdownParam.iGuestOSMode);
	sprintf(szTemp, "%s%s", S_GuestOSMode, szBuf);
	WriteOneLine(szTemp, 2);

	WriteOneLine(EndSection, 1);
	CloseFile();
	return 1;
}

//************************************************************************//
//  Parameters:
//      pszText
//          [in] The text written to configuration file.
//      uReturnNum
//          [in] The number of new lines written to configuration file.
//  Return value:
//      If the configuration is not opened, return 0.
//  Description:
//      Print text (pszText), and then print (uReturnNum) new lines.
//************************************************************************//
int CConfigFile::WriteOneLine(char *pszText, UINT uReturnNum)
{
	UINT uSize = 0;
	char pszTemp[2] = {0};
	UINT i = 0;

	if (m_pFilePointer == NULL)
		return 0;

	pszTemp[0] = '\n';
	pszTemp[1] = 0;

	if (pszText != NULL )
	{
		uSize = strlen(pszText);
		fwrite(pszText, sizeof(char), uSize, m_pFilePointer);
	}

	for (i = 0; i < uReturnNum; i++)
	{
		fwrite(pszTemp, sizeof(char), 1, m_pFilePointer);
	}

	return 1;
}
