#include "UpsKernel.h"
#include 	"../Public/EventRes.h"

CUpsKernel* CUpsKernel::m_pThis = NULL;

CUpsKernel::CUpsKernel()
{
	m_pThis = this;
	m_nFileLock = -1;
	m_pSHMData = NULL;
	m_iShutdownStep = 0;
	m_writefifofd = 0;
	m_readfifofd = 0;
	RebootSys = 0;
	kvm_num = 0;
	m_kvmsetp = 0;
	memset(&m_SendPipe, 0, sizeof(PIPE_DATA));
}

CUpsKernel::~CUpsKernel()
{
}

int CUpsKernel::Initialize()
{
	if (m_public.IsServiceActive(LOCKFILE_SERVICE))
	{
		printf("Warning: The Simple Shutdown Service is already in running.\n");
		return 0;
	}

	if (m_public.CreateDaemon())
		return 0;

	if ((m_nFileLock = m_public.SetLock(LOCKFILE_SERVICE)) == -1)
		return 0;

	signal(SIGCHLD, m_pThis->sig_usr);
	signal(SIGUSR1, m_pThis->sig_usr);

	if (!StartProcess(PROCESS_UPSAGENT))
		return 0;

	m_public.DelayInUsec(1, 0);

	if (InitializeService() == 0)
		return 0;

	return 1;
}

int CUpsKernel::InitializeService()
{	
	m_nService = 1;
	m_nACFail  = 0;
	m_nFirstBattLow = 1;
	m_nFirstACFail = 1;
	m_nFirstCommEst = 1;

	m_public.CreateSM(IPCKEY_SM_ONE, sizeof(SHM_DATA));
	{
		char *pt = NULL;
		pt = (char *) malloc(sizeof(SHM_DATA));

		if (pt == NULL)
			return 1;

		memset(pt, 0, sizeof(SHM_DATA));
		m_public.SetSMData(IPCKEY_SM_ONE, pt, sizeof(SHM_DATA));

		free(pt);
	}

	if (m_public.InitSems() < 0)
		return 0;

	while (m_readfifofd <= 0)
	{
		m_readfifofd = m_public.OpenFifo(FIFO_AGENTTOKERNEL, READ_MODE);
	}

	if (m_readfifofd < 0)
		return 0;

	memset(&m_SendPipe, 0, sizeof(PIPE_DATA));
	memset(&m_RecvPipe, 0, sizeof(PIPE_DATA));
	m_fileConfig.GetShutdownParameter(&m_SDParam);

	return 1;
}

void CUpsKernel::Run()
{
	int nUPSShutdown = 0;
	int nShutdown = 0;

	/* event start agent */
	m_event.WriteEventLog(pStrEvent[0]);
	if (m_SDParam.iSendMsg)
		m_event.BroadcastMessage(pStrEvent[0]);
	if (0 == m_SDParam.iKVMMode && 1 != m_SDParam.iShutdownMode)
	{
		StartKVM(); // resume saved virsh-gust 
		sleep(10);
		if (1 == m_SDParam.iGuestOSMode)  //allen20110916
			DelKVMSave();

	}
	while (m_nService)
	{	
		if (!nUPSShutdown)
		{
			if (CheckEvent())
			{
				if (!DoDelay())
					nUPSShutdown = 1;
			}
		}
		else
		{
			if (DoShutdownLogic())
			{
				if (m_SDParam.iShutdownMode != ShutdownMode_DEFAULT)
				{
					nUPSShutdown = 0;
					m_event.WriteEventLog(pStrEvent[10]);
					if (m_SDParam.iSendMsg)
						m_event.BroadcastMessage(pStrEvent[10]);

					system("killall -s SIGUSR2 ssdDaemon");
					m_public.DelayInUsec(3,0);
					HibernationSystem();
					m_public.DelayInUsec(3,0);
				}
				else
				{
					nShutdown = 1;	
					break;
				}
			}	
		}
		m_public.DelayInUsec(0, 100000);
	}

	TermService();

	m_public.CloseFifo(m_readfifofd);
	m_public.CloseFifo(m_writefifofd);
	/* m_public.CloseFifo(m_writefifoTmp); */
	m_public.UnlinkFifo(FIFO_AGENTTOKERNEL);
	m_public.UnlinkFifo(FIFO_KERNELTOAGENT);
	m_public.DestorySM(IPCKEY_SM_ONE);

	m_public.UnsetLock(m_nFileLock, LOCKFILE_SERVICE);
	unlink(LOCKFILE_SERVICE);

	/* event agent stop */
	m_event.WriteEventLog(pStrEvent[1]);
	if (m_SDParam.iSendMsg)
		m_event.BroadcastMessage(pStrEvent[1]);

	if (nShutdown)
	{
		/* event Shutdown */
		m_event.WriteEventLog(pStrEvent[7]);
		if (m_SDParam.iSendMsg)
			m_event.BroadcastMessage(pStrEvent[7]);
		if (RebootSys != 1)
			ShutdownSystem();
		else
		    RebootSystem();
	}
}

void CUpsKernel::TermService()
{
	int 	i = 0;

	for (i = 0; i < 32; i++)
	{
		signal(i, SIG_IGN);
	}

	if (fork() == 0)
	{
		for (i = 0; i < 128; i++)
		{
			close(i);
		}

		execlp("killall", "killall", "-USR1", BIN_FILE_AGENT, NULL);	
		_exit(0);
	}

	m_public.DelayInUsec(0, 500000);
	m_nService = 0;
}

//***********************************************************************************************//
//  Return value:
//      If event Battery Low or AC fail event happened first time, return 1; otherwise return 0;
//  Description:
//      Check the UPS status (AC Fail, AC Fail Recover and Battery Low Event),
//      and the communication event (establish communication, lost communication);
//***********************************************************************************************//
int CUpsKernel::CheckEvent()
{
	int iRet = 0;
	int nSHMLen;

	// willren added 2009-8-26 10:10:19
	// free m_pSHMData before assign it with new memory
	if (m_pSHMData)
		free(m_pSHMData);

	m_pSHMData = (PSHM_DATA) m_public.GetSMData(IPCKEY_SM_ONE,&nSHMLen);			

	if (m_pSHMData != NULL && m_pSHMData->uDataAvailable == 1)
	{
		if (m_nFirstCommEst)
		{
			sleep(1);

			m_SDParam.AutoMode = m_pSHMData->uAutoMode;
			m_SDParam.uPortType = m_pSHMData->uPortType;
			m_SDParam.uComPort = m_pSHMData->uPortNum;
			m_fileConfig.SetShutdownParameter(m_SDParam);

			/* event communicate establish */
			m_event.WriteEventLog(pStrEvent[4]);

			if (m_SDParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[4]);

			m_nFirstCommEst = 0;
		}
#if 1
		if (m_pSHMData->Shutdownsys == 99)
			return 1;

		if (m_pSHMData->RebootSys == 99)
		{
			RebootSys = 1;
			return 1;
		}
#endif
		m_dwUPSStatus = m_pSHMData->dwUPSStatus;

		if (CheckACFail())
		{
			if (m_nFirstACFail)
			{	
				/* event ACFail */
				m_event.WriteEventLog(pStrEvent[2]);

				if (m_SDParam.iSendMsg)
					m_event.BroadcastMessage(pStrEvent[2]);

				m_nFirstACFail = 0;
			}
			iRet = 1;
		}

		if (CheckBattLow())
		{
			if (m_nFirstBattLow)
			{
				/* event Battery Low */
				m_event.WriteEventLog(pStrEvent[3]);

				if (m_SDParam.iSendMsg)
					m_event.BroadcastMessage(pStrEvent[3]);

				m_nFirstBattLow = 0;
			}
			iRet = 1;
		}
	}
	else
	{
		if (!m_nFirstCommEst)
		{
			/* event Communicate Lost */
			m_event.WriteEventLog(pStrEvent[5]);
			if (m_SDParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[5]);

			m_nFirstCommEst = 1;
		}	
	}

	if (m_pSHMData)
	{
		free(m_pSHMData);
		m_pSHMData = NULL;
	}
	return iRet;
}

//*************************************************************************//
//  Return value:
//      If event AC fail event happened, return 1; otherwise return 0.
//  Description:
//      Check whether the AC Fail or AC Fail Recover event happened or not.
//*************************************************************************//
int CUpsKernel::CheckACFail()
{
	if (m_dwUPSStatus & UPS_STATUS_ACFAIL)
	{
		if (!m_nACFail)
		{
			m_nACFail = 1;
			m_tACFailStart = time(NULL);
		}
		return 1;
	}
	else if (m_nACFail)
	{
		/* event AC Fail Recover */
		m_event.WriteEventLog(pStrEvent[6]);
		if (m_SDParam.iSendMsg)
			m_event.BroadcastMessage(pStrEvent[6]);

		m_nFirstACFail = 1;
		m_nACFail = 0;
		m_tACFailStart = 0;
	}

	return 0;
}

//*****************************************************************************//
//  Return value:
//      If the event Battery Low event happened, return 1; otherwise return 0.
//  Description:
//      Check whether the Battery Low event happened or not.
//*****************************************************************************//
int CUpsKernel::CheckBattLow()
{
	if (m_dwUPSStatus & UPS_STATUS_BATTLOW)
	{
		if (m_nACFail)
		{
			m_nACFail = 0;
			m_tACFailStart = 0;
		}
		return 1;		
	}

	return 0;
}

//*****************************************************************************//
//  Return value:
//      If the AC Fail delay time has not passed, return 1; otherwise return 0.
//  Description:
//      When AC fail happens, it delays a certain time.
//*****************************************************************************//
int CUpsKernel::DoDelay()
{
	time_t tNow;
	time_t uPassedSeconds;

	if (m_nACFail)
	{
		tNow = time(NULL);
		uPassedSeconds = tNow - m_tACFailStart;
		if (uPassedSeconds >= (time_t)m_SDParam.uACFailDelayTime)
			return 0;

		return 1;
	}
	else
		return 0;
}

//*******************************************************************************************************//
//  Return value:
//      If the shutdown UPS command was sent successfully, and the external command delay time passed,
//      return 1; otherwise return 0.
//  Description:
//      When Battery Low event happened or AC Fail delay time passed, it will enter this shutdown logic.
//      Firstly execute external command which set by user, and send shutdown command to Module UPSAgent
//      through FIFO, and then delay the executing external command delay time.
//*******************************************************************************************************//
int CUpsKernel::DoShutdownLogic()
{
	static int nRespSN = 0;
	time_t tNow = time(NULL);
	int nRetFIFO;

	if (m_iShutdownStep == 0)
	{
		m_SendPipe.nCmdType = REQUEST_SN;
		m_SendPipe.nValue = m_SDParam.uShutdownNeedTime + m_SDParam.uExCmdNeedTime;
		m_writefifofd = m_public.OpenFifo(FIFO_KERNELTOAGENT, WRITE_MODE);
		nRetFIFO = m_public.SendFIFOData(m_writefifofd, &m_SendPipe);

		/* ExecuteExternCmd */
		m_event.WriteEventLog(pStrEvent[8]);
		if (m_SDParam.iSendMsg)
			m_event.BroadcastMessage(pStrEvent[8]);

		StartProcess(m_SDParam.szExternCmd);
		m_tStartShutdownTime = time(NULL);
		m_iShutdownStep = 1;
	}

	if (m_iShutdownStep == 1 && m_kvmsetp == 0)
	{
		if (0 == m_SDParam.iKVMMode && 1 != m_SDParam.iShutdownMode)   //allen20110916
		{
			m_event.WriteEventLog(pStrEvent[11]);
			if (m_SDParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[11]);
			int kvm_state = 0; 
			int count = 0;
			kvm_state = GetKVMState();
			while (kvm_state != 0 && count < 5)
			{ 
				CloseKVM();
				kvm_state = GetKVMState();	
				count ++;
				sleep(5);
			}
			m_kvmsetp = 1;
		}
	}

	if (m_iShutdownStep == 1)
	{
		if (m_public.ReadFIFOData(m_readfifofd, &m_RecvPipe) == 1)
		{
			if (m_RecvPipe.nCmdType == RESPONSE_SN)
				nRespSN = 1;
		}

		if (nRespSN == 0)
			return 0;

		if (((time_t)(m_SDParam.uExCmdNeedTime+m_tStartShutdownTime)) > tNow)
			return 0;
	}

	m_iShutdownStep = 0;
	m_tStartShutdownTime = 0;

	return 1;
}

int CUpsKernel::StartProcess(char *pszProcessName)
{
	int i;

	if ((pszProcessName == NULL) || !strlen(pszProcessName))
		return 0;

	for (i = 0; i < 32; i++)
	{
		if ((i != SIGCHLD) && (i != SIGUSR1))
			signal(i, SIG_IGN);
	}

	if (fork() == 0)
	{
		for (i = 0; i < 128; i++)
		{
			close(i);
		}

		i = open("/dev/null", O_RDWR);
		if (i < 0)
		{
			_exit(1);
			return(-1);
		}

		dup2(i, 1);
		dup2(i, 2);

		system(pszProcessName);
		_exit(0);
	}

	return 1;
}

void CUpsKernel::ShutdownSystem()
{
	char szbuf[32];

	memset(szbuf, 0, sizeof(szbuf));
	sprintf(szbuf, "%s", SHUTDOWNCMD);
	system(szbuf);
}

void CUpsKernel::RebootSystem()
{
	char szbuf[32];

	memset(szbuf, 0, sizeof(szbuf));
	sprintf(szbuf, "%s", REBOOTCMD);
	system(szbuf);
}

void CUpsKernel::HibernationSystem()
{
	char szbuf[32];

	memset(szbuf, 0, sizeof(szbuf));
	//sprintf(szbuf, "%s", Hibernation);
	sprintf(szbuf, "%s", Hibernation_centos7);  //應客戶要求修改 by leon 20170328
	system(szbuf);
}

int  CUpsKernel::Systembit()
{
	int gust_val, rtn;
	int fd[2];
	int pid;
	char line[10] = {0};

	if (pipe(fd) < 0)
	{
		perror("pipe");
		exit(1);
	}
	if ((pid = fork()) < 0)
	{
		perror("fork");
		exit(1);
	}
	if (pid > 0)
	{
		close(fd[1]);
		wait(NULL);
		gust_val = read(fd[0], line, 10);
		line[4] = '\0';
		rtn = strcmp("i686", line);
		if (rtn == 0)
		{
			close(fd[0]);
			return 0;
		}

		close(fd[0]);
		return 1;
	}
	else
	{
		close(fd[0]);
		int tmp = dup(1); //backup stdout
		dup2(fd[1], 1); //stdout redirectory 
		system("uname -m");
		dup2(tmp, 1);
		close(fd[1]);
		_exit(0);
	}

	return gust_val;
}

int CUpsKernel::GetKVMState()
{	
	int gust_val;
	int fd[2];
	int pid;
	char line[512] = {0};

	if (pipe(fd) < 0)
	{
		perror("pipe");
		exit(1);
	}
	if ((pid = fork()) < 0)
	{
		perror("fork");
		exit(1);
	}
	if (pid > 0)
	{
		close(fd[1]);
		wait(NULL);
		gust_val = read(fd[0], line, 512);
		int tmp = atoi(line);
		gust_val = tmp - 3;

		close(fd[0]);
	}
	else
	{
		close(fd[0]);
		int tmp = dup(1); //backup stdout
		dup2(fd[1], 1); //stdout redirectory 
		system("virsh list | wc -l");
		dup2(tmp, 1);
		close(fd[1]);
		_exit(0);
	}

	return gust_val;
}

void CUpsKernel::PraseStringStr(char *str)
{
	int i = 0;
	char tmp[100] = {0};
	FILE * kvmName = NULL;
	kvmName = fopen("/usr/lib/ssd/master/kvmname.txt","a+");
	if (kvmName == NULL)
		return ;

	/*---fixed bug: If using KVM, Simple Shutdown cannot shutdown guest OS.---
	Simple Shutdown gets the name of guest OS by executing command. This command is "virsh list".
        Then he saves this name to a file. This file path is "/usr/lib/ssd/master/kvmname.txt".
        And he reads this file and gets the name of guest OS, but it does not work well.
        There are five spaces between "Id" and "Name".
        This is different from the action in software.
	---*/

	while (*str != '\0')
	{
		if (*str >= '0'&& *str <= '9')
		{
			if ((*(str+1) == ' ')&&( *(str+6) != ' '))
			{
				str += 6;
				while (*str != ' ')
				{
					tmp[i++]= *str;
					str++;
				}
				fputs(tmp,kvmName);
				fputs("\n",kvmName);
				memset(tmp,0,sizeof(tmp));
				i = 0;
			}
		}
		str++;
	}

	fclose(kvmName);
	return ;
}
void CUpsKernel::PraseStringNum(char *str, int *arr)
{
	int i = 0;
	int j = 0;
	int state = 0;
	char tmp[4] = {0};

	while (*str != '\0')
	{
		if (*str >= '0'&& *str <= '9')
		{
			tmp[j] = *str;
			j++;
			if ((*(str+1) == ' ')&&(*(str+2) != ' '))
			    state = 1;
		}
		if (1 == state)
		{
			arr[i] = atoi(tmp);
			i++;
			state = 0;
			j = 0;	
			str+=15;
		}
		str++;
	}

	return ;
}

void CUpsKernel::CloseKVM()
{
	int pid;
	int kvmstate,len = 0;
	int fd[2];
	char buf[2048] = {0};
	char cmd_str[256]={0};
	char tmp[100] = {0};
	char kName[100]={0};

	int vm_id[10] = {0};
	int *p;

	FILE *kvmName = NULL;
	kvmstate = GetKVMState();
	if (kvmstate != 0)
	{
		if (pipe(fd) < 0)
		{
			perror("pipe");
			exit(1);
		}
		if ((pid = fork()) < 0)
		{
			perror("fork");
			exit(1);
		}
		if (pid > 0)
		{
			close(fd[1]);
			wait(NULL);
			read(fd[0], buf, 512);

			if (0 == m_SDParam.iGuestOSMode)
			{
				PraseStringStr(buf);

				kvmName = fopen("/usr/lib/ssd/master/kvmname.txt","r");
				if (kvmName == NULL)
					return ;

				while (fgets(tmp,100,kvmName))
				{
					len = strlen(tmp)-1;
					memcpy(kName,tmp,len);

					sprintf(cmd_str,"virsh resume %s",kName);
					system(cmd_str);

					sprintf(cmd_str,"virsh shutdown %s",kName);
					system(cmd_str);

					memset(tmp,0,sizeof(tmp));
					memset(kName,0,sizeof(kName));
					memset(cmd_str,0,sizeof(cmd_str));
				}
				fclose(kvmName);
			}
			else if (1 == m_SDParam.iGuestOSMode)
			{
				PraseStringStr(buf);

				kvmName = fopen("/usr/lib/ssd/master/kvmname.txt","r");
				if (kvmName == NULL)
					return ;

				while (fgets(tmp,100,kvmName))
				{
					len = strlen(tmp)-1;
					memcpy(kName,tmp,len);

					sprintf(cmd_str,"virsh save %s /usr/lib/ssd/master/%s.save",kName,kName);
					system(cmd_str);

					memset(tmp,0,sizeof(tmp));
					memset(kName,0,sizeof(kName));
					memset(cmd_str,0,sizeof(cmd_str));
				}
				fclose(kvmName);
			}
			close(fd[0]);

		}
		else
		{
			close(fd[0]);
			int tmp = dup(1); //backup stdout
			dup2(fd[1], 1); //stdout redirectory
			system("virsh list");
			_exit(1);
		}
	}
}

#if 0
void CUpsKernel::CloseKVM()
{
	int pid;
	int kvmstate;
	char buf[2048] = {0};
	if ((pid = fork()) < 0)
	{
		perror("fork");
		exit(1);
	}
	if (pid > 0)
	{
		wait(NULL);
		kvmstate = GetKVMState();
		if (kvmstate != 0)
		{
			int fd;
			fd = open("/usr/lib/ssd/master/kvm_start", O_RDONLY);
			int n = read(fd, buf, 2048);
			if (-1 == n)
			{
				perror("read");
				exit(1);
			}
			close(fd);

			while (kvmstate)
			{
				unlink("/usr/lib/ssd/master/kvm_start");
				system("/usr/lib/ssd/master/my_kvm.sh stop");
				kvmstate = GetKVMState();
			}
			fd = open("/usr/lib/ssd/master/kvm_start", O_WRONLY|O_TRUNC);
			n = write(fd, buf, 2048);
			if (-1 == n)
			{
				perror("read");
				exit(1);
			}
			close(fd);


		}
	}
	else
	{
		system("/usr/lib/ssd/master/my_kvm.sh stop");
		_exit(0);
	}
}
#endif

void CUpsKernel::StartKVM()
{
	int kvm_state,pid,kvmNameLen = 0;
	int counter = 0;
	int guestNum = 0;
	char cmd_str[256] = {0};
	FILE *kvmName = NULL;
	char tmp[100] = {0};
	char kName[100] = {0};
	int len = 0;

	DIR *p_dir;
	struct dirent *p_dirent;

	if (0 == m_SDParam.iGuestOSMode)
	{
        kvmName = fopen("/usr/lib/ssd/master/kvmname.txt","r");
        if (kvmName == NULL)
            return ;

        while (fgets(tmp,100,kvmName))
        {
            len = strlen(tmp)-1;
            memcpy(kName,tmp,len);

            sprintf(cmd_str,"virsh start %s",kName);
            system(cmd_str);
            memset(tmp,0,sizeof(tmp));
            memset(kName,0,sizeof(kName));
        }
        fclose(kvmName);
		system("rm -rf /usr/lib/ssd/master/kvmname.txt");
		return ;
	}
	else if ( 1 == m_SDParam.iGuestOSMode)
	{
		for (counter = 0; counter < 20; counter++)
		{
			if ((p_dir = opendir("/usr/lib/ssd/master/")) == NULL)
			{
				printf("cannot open the directory\n");
				return ;
			}

			while (p_dirent =readdir(p_dir))
			{
				if ((strstr(p_dirent->d_name,".save")) != NULL)
				{	
					kvm_num = kvm_num + 1;
					sprintf(cmd_str,"virsh restore /usr/lib/ssd/master/%s",p_dirent->d_name);
					system(cmd_str);				
				}
			}
			closedir(p_dir);
			sleep(1);
			if (kvm_num == GetKVMState())
			    break;
			kvm_num = 0;
		}
		system("rm -rf /usr/lib/ssd/master/kvmname.txt");
		return ;
	}
}
void CUpsKernel::DelKVMSave()
{
	int tmp;
	DIR *p_dir;
	struct dirent *p_dirent;
	char cmd_str[100] = {0};

	if ((p_dir = opendir("/usr/lib/ssd/master/")) == NULL)
	{
		printf("cannot open the directory\n");
		return ;
	}
	while (p_dirent =readdir(p_dir))
	{
		if ((strstr(p_dirent->d_name,".save")) != NULL)
		{	
			sprintf(cmd_str,"rm -rf /usr/lib/ssd/master/%s",p_dirent->d_name);
			system(cmd_str);
		}	
	}

}

#if 0
void CUpsKernel::StartKVM()
{
	if (access("/usr/lib/ssd/master/kvm_start", F_OK) != -1)
		system("/usr/lib/ssd/master/my_kvm.sh start");
}
#endif

void CUpsKernel::sig_usr(int sig)
{
	int status;

	if (sig == SIGCHLD)
		wait(&status);

	if (sig == SIGUSR1)
		m_pThis->TermService();

}

int main(int argc, char **argv)
{
	CUpsKernel 	upsKernel;

	if (upsKernel.Initialize())
		upsKernel.Run();

	return 0;
}
