#include "UPSAgent.h"
#include "../Public/EventRes.h"

CUPSAgent* CUPSAgent::m_pThis = NULL;


CUPSAgent::CUPSAgent()
{
	m_iRunAgent = 1;
	m_uCommPort = 0;
	m_dwUPSStatus = 0;
	m_iACFail = 0;
	m_tACFailStart = 0;
	Commandexec = 0;
	iConnect = 0;

	m_readfifofd = 0;
	m_writefifofd = 0;

	m_SendPipe.nCmdType = RESPONSE_SN;
	m_SendPipe.nValue = 0;
	memset(&m_RecvPipe, 0, sizeof(PIPE_DATA));

	m_iAutoPort = AUTO_SEL;
	m_pThis = this;
	m_cfg.GetShutdownParameter(&m_AutoSDParam);

	if (m_AutoSDParam.uPortType == 0)
		m_ups.m_devType = USB;
	else
		m_ups.m_devType = COM;
}

CUPSAgent::~CUPSAgent()
{
	m_iRunAgent = 0;
}

//************************************************//
//  Do initialize of the daemon service ssdDaemon.
//  Return 1, succeed; 0, failed.
//************************************************//
int CUPSAgent::Initialize()
{
	if (m_AutoSDParam.iUpsCommMode == LIBUSB)
		m_ups.Initialize_libusb();

	if (m_AutoSDParam.iUpsCommMode == HID)
		m_ups.Initialize_hid();

	signal(SIGUSR2, m_pThis->sig_usr);
	while (m_readfifofd <= 0)
	{
		m_readfifofd = m_public.OpenFifo(FIFO_KERNELTOAGENT, READ_MODE);
	}

	if (m_readfifofd > 0)
		return 1;
	else
		return 0;
}

int CUPSAgent::ClearReadValue()
{
	m_ups.m_nOmronStatus = 0;
	m_SHMData.RebootSys = 0;
	m_SHMData.Shutdownsys = 0;
	return 0;
}

//*******************************************************************************************//
//  It is the main function of module UPSAgent.
//  (1) Communication with UPS: if lost communication, try to establish with UPS.
//          And if communicate OK, send Command “Q1”to UPS to get the current status of UPS,
//          and then send the UPS status to UPSKenel through share memory.
//  (2) if receive shutdown command from UPSKenel, send “Sn” command to UPS.
//*******************************************************************************************//
int CUPSAgent::ScanPort()
{
	int i, maxi, maxfd, listenfd, connfd, sockfd;
	int nready, client[FD_SETSIZE];
	int iRet = 0;
	fd_set rset, allset; // Descriptor sets
	int nRetFifo, nRetSelect;	
	int nFirstScanPort = 1;
	socklen_t cliaddr_len;
	struct sockaddr_in	cliaddr;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 500000;

	/* init the socket communication with omrctl */
	listenfd = m_public.serv_create();
	if (listenfd < 0)
	{
		strerror(errno);
		exit(0);
	}

	maxfd = listenfd;		/* initialize */
	maxi = -1;			/* index into client[] array */
	for (i = 0; i < FD_SETSIZE; i++)
	{
		client[i] = -1;	/* -1 indicates available entry */
	}
	FD_ZERO(&allset);
	FD_SET(listenfd, &allset);

	while (m_iRunAgent)
	{
		rset = allset;	/* structure assignment */
		nready = select(maxfd+1, &rset, NULL, NULL, &tv);
		if (nready < 0)
		{
			perror("select error");
			exit(0);
		}
		if (nready > 0)
		{
			if (FD_ISSET(listenfd, &rset))  /* new client connection */
			{
				cliaddr_len = sizeof(cliaddr);
				connfd = m_public.Accept(listenfd, (struct sockaddr *)&cliaddr, &cliaddr_len);

				for (i = 0; i < FD_SETSIZE; i++)
				{
					if (client[i] < 0)
					{
						client[i] = connfd; /* save descriptor */
						break;
					}
				}
				if (i == FD_SETSIZE)
				{
					fputs("too many clients\n", stderr);
					exit(1);
				}

				FD_SET(connfd, &allset);	/* add new descriptor to set */
				if (connfd > maxfd)
					maxfd = connfd; /* for select */
				if (i > maxi)
					maxi = i;	/* max index in client[] array */

				if (--nready == 0)
					continue;	/* no more readable descriptors */
			}
			for (i = 0; i <= maxi; i++) 
			{	/* check all clients for data */
				if ( (sockfd = client[i]) < 0)
					continue;
				if (FD_ISSET(sockfd, &rset)) 
				{
					if ( (ReadSocket(sockfd, iConnect)) == 0) 
					{
						/* connection closed by client */
						close(sockfd);
						FD_CLR(sockfd, &allset);
						client[i] = -1;
					} 
					if (--nready == 0)
						break;	/* no more readable descriptors */
				}
			}
		}
		if (m_AutoSDParam.uPortType != m_ups.m_devType || 
				m_AutoSDParam.uComPort != m_ups.m_nPortNum || 
				m_AutoSDParam.AutoMode != m_iAutoPort)
		{
			if (iConnect == 1)
			{
				memset(&(m_SHMData), 0, sizeof(m_SHMData));
				m_SHMData.uAutoMode = m_AutoSDParam.AutoMode;
#if 1
				//colin
				if (m_ups.m_nOmronStatus != 0) 
				{
					if (m_ups.m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
						m_SHMData.RebootSys = 99;
					else 
						m_SHMData.Shutdownsys = 99;
				}
#endif
				m_public.SetSMData(IPCKEY_SM_ONE, &m_SHMData, sizeof(SHM_DATA));
			}
			iConnect = 0;
		}

		if (!iConnect)
		{
			if (0 == m_AutoSDParam.AutoMode)
				iConnect = m_ups.OpenSettedCom(m_AutoSDParam.uComPort);
			else if (AUTOMODE_ALL_USB_WITH_COM == m_AutoSDParam.AutoMode)
				iConnect = m_ups.ConnectUPS();
			else if (AUTOMODE_ALL_USB_WITHOUT_COM == m_AutoSDParam.AutoMode)
				iConnect = m_ups.ConnectUPSOnlyUSB();	

			if (nFirstScanPort == 1 )
			{
				if (!iConnect)
				{
					m_event.WriteEventLog(pStrEvent[9]);
					if (m_AutoSDParam.iSendMsg)
						m_event.BroadcastMessage(pStrEvent[9]);
				}
				nFirstScanPort = 0;
			}
			if (iConnect)
			{
				m_iAutoPort = m_AutoSDParam.AutoMode;
				m_AutoSDParam.uPortType = m_ups.m_devType;
				m_AutoSDParam.uComPort = m_ups.m_nPortNum;
			}
		}
		else
		{
			nRetFifo = m_public.ReadFIFOData(m_readfifofd, &m_RecvPipe);
			if (nRetFifo == 1 && m_RecvPipe.nCmdType == REQUEST_SN)
			{
				m_ups.UPSCmdSN(m_RecvPipe.nValue);
				m_writefifofd = m_public.OpenFifo(FIFO_AGENTTOKERNEL, WRITE_MODE);
				nRetFifo = m_public.SendFIFOData(m_writefifofd, &m_SendPipe);
			}

			iConnect = m_ups.PollingUPS();

			if (!iConnect)
			{
				memset(&(m_SHMData), 0, sizeof(m_SHMData));
				m_SHMData.uAutoMode = m_AutoSDParam.AutoMode;
				//colin
				if (m_ups.m_nOmronStatus != 0)
				{
					if (m_ups.m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
						m_SHMData.RebootSys = 99;
					else 
						m_SHMData.Shutdownsys = 99;
				}
				m_public.SetSMData(IPCKEY_SM_ONE, &m_SHMData, sizeof(SHM_DATA));
			}
			else
			{	
				m_SHMData.uDataAvailable = 1;
				m_SHMData.uAutoMode = m_AutoSDParam.AutoMode;
				m_SHMData.uPortType = m_ups.m_devType;
				m_SHMData.uPortNum = m_ups.m_nPortNum;
				m_SHMData.dwUPSStatus = m_ups.m_dwUPSStatus;	
				//colin
				if (m_ups.m_nOmronStatus != 0)
				{
					if (m_ups.m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
						m_SHMData.RebootSys = 99;
					else 
						m_SHMData.Shutdownsys = 99;
				}

				m_public.SetSMData(IPCKEY_SM_ONE, &m_SHMData, sizeof(SHM_DATA));
			}		
		}

		if (!iConnect)
			m_public.DelayInUsec(0, 100000);

		m_public.DelayInUsec(0, 100000);
		m_public.DelayInUsec(0, 100000);

	}

	m_ups.ClosePort();
	m_public.Close(listenfd);
	m_public.CloseFifo(m_readfifofd);
	m_public.DestorySM(IPCKEY_SM_ONE);
	m_public.UnlinkFifo(FIFO_KERNELTOAGENT);
	m_iRunAgent = 0;
	m_iAutoPort = AUTO_SEL;

	return iRet;
}

void CUPSAgent::Terminate()
{
	m_iRunAgent = 0;	
}

void CUPSAgent::sig_usr(int sig)
{
	if (sig == SIGUSR2)
	{
		m_pThis->Commandexec = 0;
		m_pThis->ClearReadValue();
		m_pThis->m_cfg.GetShutdownParameter(&(m_pThis->m_AutoSDParam));
	}
}

int CUPSAgent::ReadSocket(int socketfd, int iConnect)
{
	int rtn = 0;
	char buf[sizeof(Socketdata)] = {0};
	rtn = m_public.Read(socketfd,buf,sizeof(Socketdata));

	memset(&Socketdata, 0, sizeof(Socketdata));
	memcpy(&Socketdata, buf, sizeof(Socketdata)); //convert string to structure

    /*---fixed bug: OS shutdowns without AC fail, after port scan.---
        //Check "Socketdata.uCmd" is not 0. It is not enough.
        //The size of "Socketdata.uCmd" is 4 byte.
        //Only 1 byte data of "Socketdata.uCmd" is used.
        //Other 3 byte data should be 0.
        ---*/
	if ((Socketdata.uCmd != 0) && ((Socketdata.uCmd & 0xFFFFFF00)==0))
	{
		if (!iConnect)
		{
			memset(&Socketdata, 0, sizeof(Socketdata));
			Socketdata.uPara3 = ERR_COMLST;
			memcpy(buf,&Socketdata,sizeof(Socketdata)); // convert struct to string
			rtn = m_public.Write(socketfd,buf,sizeof(Socketdata));
		}
		else if (1 == Commandexec)
		{
			memset(&Socketdata, 0, sizeof(Socketdata));
			Socketdata.uPara4 = ERR_COMMEXEC;
			memcpy(buf,&Socketdata,sizeof(Socketdata)); // convert struct to string
			rtn = m_public.Write(socketfd,buf,sizeof(Socketdata));
		}
		else // colin
		{
			m_ups.m_nOmronStatus = Socketdata.uCmd;
			m_ups.m_nRestartTime = Socketdata.uPara2;
			Commandexec = 1;
		}
	}

	return 0;
}

